#!/usr/bin/env python
# matrixctl
# Copyright (c) 2020  Michael Sasser <Michael@MichaelSasser.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import sys
import configparser
import functools

from typing import Tuple
from pathlib import Path
from logging import debug, fatal

from matrixctl import HOME

__author__: str = "Michael Sasser"
__email__: str = "Michael@MichaelSasser.org"


class ConfigFileError(Exception):
    pass


def error_if_not_available(method):
    """A wrapper, that checks, if the configuration is available and returns
    it. If the information is not available. it throws an error.
    """

    @functools.wraps(method)
    def wrapper_error_if_not_available(*args, **kwargs):
        try:
            return method(*args, **kwargs)
        except (KeyError, TypeError):
            fatal(
                f"Please check your config file: "
                f"{str(method.__name__).split('_')[0].upper()} "
                f"({str(method.__name__).split('_')[1]} entry)"
            )
            sys.exit(1)

    return wrapper_error_if_not_available


def none_if_not_available(method):
    """A wrapper, that checks, if the configuration is available and returns
    it. If the information is not available. it returns None.
    """

    @functools.wraps(method)
    def wrapper_none_if_not_available(*args, **kwargs):
        try:
            return method(*args, **kwargs)
        except (KeyError, TypeError):
            debug(f"Config entry not available for: {method.__name__}")

            return None

    return wrapper_none_if_not_available


# ToDo: Is the token length always the same? should be. If yes: check token
# length to prevent copy error.
class Config:
    FILE_PATH: Tuple[Path] = (
        Path("/etc/matrixctl/config"),
        Path(f"/{HOME}/.config/matrixctl/config"),
    )

    def __init__(self):
        debug("Loading Config file(s)")

        self.__check_path(self.__class__.FILE_PATH)

        self.config = configparser.ConfigParser()
        self.config.read(self.__class__.FILE_PATH)

        # Debugging
        self.__debug_output()

        # [ANSIBLE]
        try:
            self.config_ansible = self.config["ANSIBLE"]
        except KeyError:
            raise ConfigFileError(
                "Please check your configuration file. "
                "The [DEFAULT] section is mandatory."
            )
        except TypeError:
            raise ConfigFileError(
                "To use this program you need to have a config file in"
                '/etc/matrixctl/config" or in "~/.config/matrixctl/config".'
            )

        # [API]
        try:
            self.config_api = self.config["API"]
        except KeyError:
            self.config_api = None

        # [SERVER]
        try:
            self.config_server = self.config["SERVER"]
        except KeyError:
            self.config_server = None

    def __debug_output(self):
        for section in self.config.sections():
            debug(f"[{section}]")

            for entry in self.config[section]:
                if entry == "token":
                    length = len(self.config[section][entry])
                    debug(f"  ├─  {entry} := **HIDDEN (Length={length})**")
                else:
                    debug(f"  ├─  {entry} := {self.config[section][entry]}")
            debug("  ┴")

    @staticmethod
    def __check_path(file_path):
        path_exists: bool = False

        for p in file_path:
            if p.exists():
                path_exists = True

        if not path_exists:
            fatal(
                "To use this program you need to have a config file in "
                '/etc/matrixctl/config" or in "~/.config/matrixctl/config".'
            )
            sys.exit(1)

    @property
    def ansible_path(self) -> Path:
        return Path(self.config_ansible["MatrixDockerAnsibleDeployPath"])

    @property
    @error_if_not_available
    def api_token(self) -> str:
        return self.config_api["Token"]

    @property
    @error_if_not_available
    def api_domain(self) -> str:
        return self.config_api["Domain"]

    @property
    @none_if_not_available
    def server_cfg(self) -> str:
        return self.config_server["AnsibleCfg"]

    @property
    @none_if_not_available
    def server_play(self) -> str:
        return self.config_server["AnsiblePlaybook"]

    @property
    @none_if_not_available
    def server_tags(self) -> str:
        return self.config_server["AnsibleTags"]


# vim: set ft=python :



#!/usr/bin/env python
# matrixctl
# Copyright (c) 2020  Michael Sasser <Michael@MichaelSasser.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# import configparser
import datetime
from logging import debug

import git

from .config_handler import Config

__author__: str = "Michael Sasser"
__email__: str = "Michael@MichaelSasser.org"


def git_pull(cnf: Config):
    # Get the last pulled datetime
    repo = git.Repo(cnf.ansible_path)

    assert not repo.bare
    heads = repo.heads
    master = heads.master

    log = master.log()
    last = datetime.datetime.fromtimestamp(log[-1].time[0])
    debug(f"Git: last update: {last}")

    # Pull request and "log" since last pulled
    g = git.cmd.Git(cnf.ansible_path)
    g.pull()
    print(
        g.log(
            f"--since={str(last)}",
            # f"--since={str(last.date())}",
            "--pretty=%as |%<(15) %an | %s",
        )
    )


# vim: set ft=python :



#!/usr/bin/env python
# matrixctl
# Copyright (c) 2020  Michael Sasser <Michael@MichaelSasser.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import string
import getpass
import secrets
from typing import Optional
from tabulate import tabulate
from .ansible_handler import ansible_synapse
from .config_handler import Config
from .ssh_handler import Ssh

__author__: str = "Michael Sasser"
__email__: str = "Michael@MichaelSasser.org"


SPECIAL = "!\"§$%&/()=?.,;:_-#'+*~{}[]`´°^@<>|\\"
ALPHABET = string.ascii_letters + string.digits + SPECIAL


BOTS = {"whatsapp_", "whatsappbot", "telegram_", "telegrambot"}


def ask_password() -> Optional[str]:
    passwd: str = "a"
    passwd2: str = "b"

    while passwd != passwd2:
        passwd = getpass.getpass()
        passwd2 = getpass.getpass("Password (again): ")

    return passwd if passwd == passwd2 else None


def gen_password() -> str:
    while True:
        password = "".join(secrets.choice(ALPHABET) for i in range(16))

        # pylint: disable=chained-comparison

        if (
            any(c.islower() for c in password)
            and any(c.isupper() for c in password)
            and sum(c.isdigit() for c in password) >= 3
            and 1 <= sum(c in SPECIAL for c in password) <= 3
        ):
            return password


def ask_question(question: str = "Is everything ok?") -> True:
    question += " [y/n]"

    while (answer := input(question).lower()) not in ("y", "j", "n",):  # noqa
        pass

    return answer in ("y", "j")


def adduser(arg, cfg: Config, adminapi):
    """Adds a User to the synapse instance"""

    while True:
        passwd_generated: bool = False

        if arg.passwd is None:
            arg.passwd = ask_password()

        if arg.passwd == "":
            arg.passwd = gen_password()
            passwd_generated = True

        print(f"Username: {arg.user}")

        if passwd_generated:
            print(f"Password (generated): {arg.passwd}")
        else:
            print(f"Password: **HIDDEN**")
        print(f"Admin:    {'yes' if arg.admin else 'no'}")

        answer = ask_question()

        if answer:
            break
        arg.passwd = None

    if arg.ansible:
        ansible_synapse(
            [
                f"--extra-vars='username={arg.user} password={arg.passwd} admin={arg.admin}'",
                "--tags=register-user",
            ],
            cfg,
        )
    else:
        adminapi.adduser(arg.user, arg.passwd, arg.admin)


def adduser_jitsi(arg, cfg: Config, _):
    """Adds a User to the synapse instance"""

    while True:
        passwd_generated: bool = False

        if arg.passwd is None:
            arg.passwd = ask_password()

        if arg.passwd == "":
            arg.passwd = gen_password()
            passwd_generated = True

        print(f"Username: {arg.user}")

        if passwd_generated:
            print(f"Password (generated): {arg.passwd}")
        else:
            print(f"Password: **HIDDEN**")

        answer = ask_question()

        if answer:
            break
        arg.passwd = None

    with Ssh(cfg) as ssh:
        ssh.adduser(arg.user, arg.passwd)


def deluser(arg, _: Config, adminapi):
    adminapi.deluser(arg.user)


def deluser_jitsi(arg, cfg: Config, _):
    with Ssh(cfg) as ssh:
        ssh.deluser(arg.user)


def list_users(arg, cfg: Config, adminapi):
    len_domain = len(cfg.api_domain) + 1  # 1 for :
    from_user: int = 0
    users: list = []

    while True:
        lst = adminapi.list_users(from_user, show_guests=arg.guests).json()

        users += lst["users"]
        try:
            from_user = lst["next_token"]
        except KeyError:
            break

    user_list: list = []

    for user in users:
        name = user["name"][1:-len_domain]
        no_passwd_hash: bool = user["password_hash"] == ""
        deactivated: bool = bool(int(user["deactivated"]))
        admin: bool = bool(int(user["admin"]))
        guest: bool = bool(int(user["is_guest"]))

        # if no_bots and any([name.startswith(bot) for bot in BOTS]):
        #     continue

        if arg.no_bots and no_passwd_hash:
            continue

        user_list.append((name, deactivated, admin, guest,))
    print(
        tabulate(
            user_list,
            headers=("Name", "Deactivated", "Is Admin", "Is Guest"),
            tablefmt="psql",
        )
    )


# vim: set ft=python :



#!/usr/bin/env python
# matrixctl
# Copyright (c) 2020  Michael Sasser <Michael@MichaelSasser.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import sys
from typing import NamedTuple, Optional
from logging import debug, error

from paramiko import SSHClient
from paramiko.channel import ChannelFile

from .config_handler import Config


__author__: str = "Michael Sasser"
__email__: str = "Michael@MichaelSasser.org"


class SSHResponse(NamedTuple):
    stdin: Optional[str]
    stdout: Optional[str]
    stderr: Optional[str]


class Ssh:

    JID_EXT: str = "matrix-jitsi-web"

    def __init__(self, cfg: Config, address: str = None):
        self.config: Config = cfg

        if address is None:
            address = f"matrix.{self.config.api_domain}"
        self.address: str = address
        self.client: SSHClient = SSHClient()
        self.client.load_system_host_keys()
        self.connect()

    def connect(self):
        """Connects to the SSH server"""
        self.client.connect(self.address)

    def __enter__(self):
        """Connects to the SSH server with the "with" command"""

        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        """Closes the connection to the SSH after the namespace of the
        "with" command ends."""
        self.client.close()

    def __del__(self):
        """Clos the connection to the SSH"""
        self.client.close()

    @staticmethod
    def __str_from(f: ChannelFile) -> Optional[str]:
        try:
            return f.read().decode("utf-8").strip()
        except OSError:
            return None

    def run_cmd(self, cmd: str):
        debug(f'SSH Command: "{cmd}"')

        response = SSHResponse(
            *[self.__str_from(s) for s in self.client.exec_command(cmd)]
        )

        debug(f'SSH Response: "{response}"')

        return response

    def adduser(self, user: str, password: str):
        """Add a user to the jitsi server."""
        # register: undocumented, means: "user@JID_EXT passwd" as stdin
        # without asking for the password. (replaces adduser)
        cmd: str = f'sudo docker exec matrix-jitsi-prosody prosodyctl --config /config/prosody.cfg.lua register "{user}" {self.__class__.JID_EXT} "{password}"'

        res = self.run_cmd(cmd)

        if res.stderr.startswith("bash: 7: command not found"):
            error(
                "BUG: It's likely that you had previously installed Jitsi "
                "without auth/guest support. Please look into the "
                "configuring-playbook-jitsi.md in "
                "matrix-docker-ansible-deploy/docs. Read the paragraph "
                "about rebuilding your Jitsi installation."
            )
            sys.exit(1)

        return res

    def deluser(self, user: str):
        """Delete a user from the jitsi server."""
        cmd: str = f'sudo docker exec matrix-jitsi-prosody prosodyctl --config /config/prosody.cfg.lua deluser "{user}@{self.__class__.JID_EXT}"'

        return self.run_cmd(cmd)


# vim: set ft=python :



#!/usr/bin/env python
# matrixctl
# Copyright (c) 2020  Michael Sasser <Michael@MichaelSasser.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import os
import subprocess
from typing import Iterable, List
from .config_handler import Config

__author__: str = "Michael Sasser"
__email__: str = "Michael@MichaelSasser.org"


def ansible_server(cfg: Config) -> None:
    if cfg.server_play is not None:

        # Add ENV variable

        if cfg.server_cfg is not None:
            os.environ["ANSIBLE_CONFIG"] = cfg.server_cfg

        # Build command
        cmd: List[str] = [
            "ansible-playbook",
            cfg.server_play,
        ]

        if cfg.server_tags is not None:
            cmd.append(f"--tags={cfg.server_tags}")

        # Run command
        subprocess.run(cmd, check=True)

        # Delete ENV variable

        if cfg.server_cfg is not None:
            del os.environ["ANSIBLE_CONFIG"]


def ansible_synapse(arguments: Iterable[str], cfg: Config) -> None:
    assert isinstance(arguments, (list, tuple))

    # Build Command
    cmd: List[str] = [
        "ansible-playbook",
        "-i",
        f"{str(cfg.ansible_path)}/inventory/hosts",
        f"{str(cfg.ansible_path)}/setup.yml",
    ]
    cmd += list(arguments)

    # Run command
    subprocess.run(cmd, check=True)


# vim: set ft=python :



#!/usr/bin/env python
# matrixctl
# Copyright (c) 2020  Michael Sasser <Michael@MichaelSasser.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
import json
from logging import debug, error
from typing import Optional

import requests

from matrixctl import __version__
from .config_handler import Config

__author__: str = "Michael Sasser"
__email__: str = "Michael@MichaelSasser.org"


class Api:
    def __init__(self, config: Config):
        self.config = config
        self.url = f"https://matrix.{self.config.api_domain}"
        self.session = requests.Session()

    def adduser(self, user: str, password: str, admin: bool = False):
        """Add a user to the matrix server."""
        path = f"/users/@{user}:{self.config.api_domain}"
        payload = {
            "password": password,
            "admin": admin,
        }
        response = self.send(path, payload=payload, method="PUT")

        if response.status_code not in (201, requests.codes.ok):
            error("The User was not added.")

    def deluser(self, user: str):
        """Add a user to the matrix server."""
        path = f"/deactivate/@{user}:{self.config.api_domain}"
        payload = {
            "erase": True,
        }
        response = self.send(
            path, payload=payload, method="POST", api_version=1
        )

        if response.status_code not in (201, requests.codes.ok):
            error("The User was not added.")

    def list_users(self, from_user: int = 0, show_guests: bool = False):
        """Add a user to the matrix server."""
        path = f"/users"
        params = {
            "from": from_user,
            "guests": "true" if show_guests else "false",
        }
        response = self.send(path, method="GET", params=params)

        if response.status_code not in (201, requests.codes.ok):
            error("The request was not successful.")

        return response

    def send(
        self,
        path: str,
        params: Optional[dict] = None,
        payload: Optional[dict] = None,
        method: str = "GET",
        api_version: int = 2,
        json_payload: bool = True,
    ):
        method = method.upper()
        assert method in {"GET", "POST", "PUT", "DELETE"}

        api_path = f"/_synapse/admin/v{api_version}"
        headers: dict = {
            "User-Agent": f"matrixctl{__version__}",
            "Authorization": f"Bearer {self.config.api_token}",
        }

        if json_payload:
            headers["Content-Type"] = "application/json"

        endpoint = self.url + api_path + path

        if (
            headers["Content-Type"] == "application/json"
            and payload is not None
        ):
            payload = json.dumps(payload)

        debug(f"{endpoint=}")
        debug(f"{headers=}")
        debug(f"{params=}")
        debug(f"{payload=}")

        response = self.session.request(
            method, endpoint, headers=headers, data=payload, params=params
        )

        debug(f"{response.json()=}")

        return response


# if __name__ == "__main__":
#     from pprint import pprint  # noqa
#
#     r_path = "/users?guests=false"
#     r_params = {"limit": 3}
#     api = Api()
#     pprint(api.send(r_path, params=r_params))

# vim: set ft=python :



#!/usr/bin/env python
# matrixctl
# Copyright (c) 2020  Michael Sasser <Michael@MichaelSasser.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
from logging import debug
from .ansible_handler import ansible_synapse

__author__: str = "Michael Sasser"
__email__: str = "Michael@MichaelSasser.org"


def maintainance(_, cfg, __):
    debug(f"maintainance")
    ansible_synapse(
        ["--tags=run-postgres-synapse-janitor,run-postgres-vacuum,start"], cfg
    )


# vim: set ft=python :



#!/usr/bin/env python
# matrixctl
# Copyright (c) 2020  Michael Sasser <Michael@MichaelSasser.org>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# PYTHON_ARGCOMPLETE_OK
import sys
import argparse
from logging import debug, warning

import coloredlogs
import argcomplete

from matrixctl import __version__
from .config_handler import Config
from .api_handler import Api
from .housekeeping import maintainance
from .updating import update
from .account import adduser, deluser, list_users, adduser_jitsi, deluser_jitsi
from .provisioning import deploy


__author__: str = "Michael Sasser"
__email__: str = "Michael@MichaelSasser.org"

# API: https://github.com/matrix-org/synapse/blob/master/docs/admin_api/user_admin_api.rst


def main():
    parser = argparse.ArgumentParser()

    parser.add_argument("--version", action="version", version=__version__)
    parser.add_argument(
        "-d", "--debug", action="store_true", help="Enables debugging mode."
    )
    subparsers = parser.add_subparsers()

    ##########################################################################
    # adduser
    adduser_parser = subparsers.add_parser(
        "adduser", help="Add a new matrix user"
    )
    adduser_parser.add_argument("user", help="The Username of the new user")
    adduser_parser.add_argument(
        "-p",
        "--passwd",
        help="The password of the new user. (If you don't enter a password, "
        "you will be asked later.)",
    )
    adduser_parser.add_argument(
        "-a", "--admin", action="store_true", help="Create as admin user"
    )
    adduser_parser.add_argument(
        "--ansible", action="store_true", help="Use ansible insted of the api"
    )
    adduser_parser.set_defaults(func=adduser)

    ##########################################################################
    # adduser-jitsi
    adduser_jitsi_parser = subparsers.add_parser(
        "adduser-jitsi", help="Add a new jitsi user"
    )
    adduser_jitsi_parser.add_argument(
        "user", help="The Username of the new jitsi user"
    )
    adduser_jitsi_parser.add_argument(
        "-p",
        "--passwd",
        help="The password of the new jitsi user. (If you don't enter a "
        "password, you will be asked later.)",
    )
    adduser_jitsi_parser.set_defaults(func=adduser_jitsi)

    ##########################################################################
    # deluser
    deluser_jitsi_parser = subparsers.add_parser(
        "deluser-jitsi", help="Deletes a jitsi user"
    )
    deluser_jitsi_parser.add_argument(
        "user", help="The jitsi username to delete"
    )
    deluser_jitsi_parser.set_defaults(func=deluser_jitsi)

    ##########################################################################
    # list-users
    list_users_parser = subparsers.add_parser("list-users", help="Lists users")
    list_users_parser.add_argument(
        "-g", "--guests", action="store_true", help="Shows the users"
    )
    list_users_parser.add_argument(
        "-b", "--no-bots", action="store_true", help="Hide bots"
    )
    list_users_parser.set_defaults(func=list_users)

    ##########################################################################
    # deluser
    deluser_parser = subparsers.add_parser("deluser", help="Deletes a user")
    deluser_parser.add_argument("user", help="The username to delete")
    deluser_parser.set_defaults(func=deluser)

    ##########################################################################
    # deploy
    deploy_parser = subparsers.add_parser(
        "deploy", help="Provision and deploy"
    )
    deploy_parser.set_defaults(func=deploy)

    ##########################################################################
    # deploy
    update_parser = subparsers.add_parser(
        "update", help="Updates the ansible repo"
    )
    update_parser.set_defaults(func=update)

    ##########################################################################
    # maintainance
    maintainance_parser = subparsers.add_parser(
        "maintainance", help="Run Maintainance tasks"
    )
    maintainance_parser.set_defaults(func=maintainance)

    argcomplete.autocomplete(parser)
    args = parser.parse_args()

    coloredlogs.DEFAULT_LOG_FORMAT = (
        "%(asctime)s - %(levelname)s - %(message)s"
    )
    coloredlogs.DEFAULT_LOG_LEVEL = 0 if args.debug else 21
    coloredlogs.install()

    config = Config()
    api = Api(config)

    if args.debug:
        debug("Disabing help on AttributeError")
        warning(
            "In debugging mode help is disabled! If you don't use any "
            "attibutes, the program will throw a AttributeError like: "
            "\"AttributeError: 'Namespace' object has no attribute 'func\".'"
            " This is perfectly normal and not a bug. If you want the help "
            'in debug mode, use the "--help" attribute.'
        )
        args.func(args, config, api)
        sys.exit()

    try:
        args.func(args, config, api)
    except AttributeError:
        parser.print_help()


# vim: set ft=python :



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['matrixctl']

package_data = \
{'': ['*']}

install_requires = \
['GitPython>=3.1.0,<4.0.0',
 'argcomplete>=1.11.1,<2.0.0',
 'coloredlogs>=14.0,<15.0',
 'paramiko>=2.7.1,<3.0.0',
 'requests>=2.23.0,<3.0.0',
 'tabulate>=0.8.7,<0.9.0']

entry_points = \
{'console_scripts': ['matrixctl = matrixctl.application:main']}

setup_kwargs = {
    'name': 'matrixctl',
    'version': '0.1.3',
    'description': 'Controls a synapse oci-container instance via ansible',
    'long_description': '![GitHub](https://img.shields.io/github/license/MichaelSasser/matrixctl?style=flat-square)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/matrixctl?style=flat-square)\n![PyPI - Wheel](https://img.shields.io/pypi/wheel/matrixctl?style=flat-square)\n![GitHub release (latest SemVer)](https://img.shields.io/github/v/release/michaelsasser/matrixctl?style=flat-square)\n![GitHub Release Date](https://img.shields.io/github/release-date/michaelsasser/matrixctl?style=flat-square)\n![PyPI - Status](https://img.shields.io/pypi/status/matrixctl?style=flat-square)\n![GitHub code size in bytes](https://img.shields.io/github/languages/code-size/michaelsasser/matrixctl?style=flat-square)\n\n# MatrixCtl\n\nMatrixCtl is a python program to control, manage, provision and deploy our\nmatrix homeserver. I had a bunch of shell scripts doing that. Two weeks\nafter using them I couldn\'t remember the order in which I have to use the\narguments or which arguments where needed. It was a pain. So I decided I hack\nsomething together fast.\n\nIt is not the most elegant piece of software I wrote, but it should do the\ntrick. I will continue to port the rest of the scripts and add a few new\nfeatures.\n\nMaybe it is also useful for someone else.\n\n## Branching Model\n\nThis repository uses the\n[git-flow](https://danielkummer.github.io/git-flow-cheatsheet/index.html)\nbranching model by [Vincent Driessen](https://nvie.com/about/).\nIt has two branches with infinite lifetime:\n\n* [master](https://github.com/MichaelSasser/matrixctl/tree/master)\n* [develop](https://github.com/MichaelSasser/matrixctl/tree/develop)\n\nThe master branch gets updated on every release. The develop branch is the\nmerging branch.\n\n## Command line tool\n\nMatrixCtl as a pure commandline tool. You can use it as package, if you like,\nbut breaking changes may be introduced, even in a minor change.\n\n```\n# matrixctl\nusage: matrixctl [-h] [--version] [-d]\n              {adduser,adduser-jitsi,deluser-jitsi,list-users,deluser,deploy,update,maintainance} ...\n\npositional arguments:\n  {adduser,adduser-jitsi,deluser-jitsi,list-users,deluser,deploy,update,maintainance}\n    adduser             Add a new matrix user\n    adduser-jitsi       Add a new jitsi user\n    deluser-jitsi       Deletes a jitsi user\n    list-users          Lists users\n    deluser             Deletes a user\n    deploy              Provision and deploy\n    update              Updates the ansible repo\n    maintainance        Run Maintainance tasks\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --version             show program\'s version number and exit\n  -d, --debug           Enables debugging mode.\n```\n\n## Configuration File\n\nTo use this program you need to have this config file in\n"/etc/matrixctl/config" or in "~/.config/matrixctl/config".\n\n```toml\n[ANSIBLE]\n# The absolute path to the fully configured matrix-docker-ansible-deploy\n# playbook from https://github.com/spantaleev/matrix-docker-ansible-deploy.\n\nMatrixDockerAnsibleDeployPath="/absolut/path/to/matrix-docker-ansible-deploy"\n\n[SERVER]\n# If you have your own playbook, to provision your matrix server, you can\n# fill out this section. MatrixCtl will run this before the\n# matrix-docker-ansible-deploy playbook.\n\n# If you have a special "ansible.cfg" for your playbook, fill in the absolute\n# path to it.\n\n# AnsibleCfg="/absolute/path/to/ansible.cfg"\n\n# Fill in the absolute path to your "site.yml"\n\n# AnsiblePlaybook="/absolute/path/to/site.yml"\n\n# If you use tags to provision or configure your matrix host, you can add them\n# here. Use a comma separated string without spaces.\n\n# AnsibleTags="MyTag,MyOtherTag"\n\n[API]\n# If your matrix server is deployed, you may want to fill out the API section.\n# It enables matrixctl to run more and faster commands. You can deploy and\n# provision your Server without this section. You also can cerate a user with\n# "matrixctl adduser --ansible YourUsername" and add your privileges after\n# that.\n\n# Your domain should be something like "michaelsasser.org" without the\n# "matrix." in the front. MatrixCtl will add that, if needed. An IP-Address\n# is not enough.\n\n# Domain="domain.tld"\n\n# To use the API you need to have an administrator account. Enter your Token\n# here. If you use the riot client you will find it your user settings (click\n# on your username on the upper left corner on your browser) in the\n# "Help & About" tab. If you scroll down click next to "Access-Token:" on\n# "<click to reveal>". It will be marked for you. Copy it in here.\n\n# Token="MyMatrixToken"\n```\n\n## Semantic Versioning\n\n**After release "1.0.0"** this repository will use\n[SemVer](https://semver.org/) for its release\ncycle.\n\n**Before release "1.0.0"** it uses "0.MAJOR.MINOR_or_PATCH".\nThis means, if breaking changes are introduced, it results in a major version\nchange (e.g. "0.1.0" -> "0.2.0"). Minor changes, like new features or patches\nare bumping the last digit (e.g. "0.1.1" -> "0.1.2").\n\n## License\nCopyright &copy; 2020 Michael Sasser <Info@MichaelSasser.org>. Released under\nthe GPLv3 license.\n',
    'author': 'Michael Sasser',
    'author_email': 'Michael@MichaelSasser.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/MichaelSasser/matrixctl',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
